/* Copyright (c) 2000-2002, Jacco Bikker
 * Copyright (c) 2002-2003, Gary Dubuque
 *
 * This file is part of NP+AIML.
 * NP+AIML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * NP+AIML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with NP+AIML; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

// AIMLpadView.cpp : implementation of the CAIMLpadView class
//

#include "stdafx.h"
#include "AIMLpad.h"

#include "AIMLpadDoc.h"
#include "AIMLpadView.h"

extern CAIMLpadApp theApp;
extern int serv;

// MSAgent support stuff
#include <agtsvr.h>

extern bool StartMSAgent(char *pMSAgentFile);
extern void ShutdownMSAgent();
extern void SpeakMSAgent(LPCSTR pszA);
extern void SetMSAgentTTS(LPCSTR pszA);
extern HRESULT ShowMSAgent();
extern HRESULT HideMSAgent();

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView

IMPLEMENT_DYNCREATE(CAIMLpadView, CEditView)

BEGIN_MESSAGE_MAP(CAIMLpadView, CEditView)
	//{{AFX_MSG_MAP(CAIMLpadView)
	ON_COMMAND(ID_TOOLS_AIML_LEARN, OnAimlLearn)
	ON_COMMAND(ID_TOOLS_AIML_SORT, OnAimlSort)
	ON_COMMAND(ID_TOOLS_AIML_DUMP, OnAimlDump)
	ON_COMMAND(ID_TOOLS_AIML_REMOVE_DUPS, OnAimlRemoveDups)
	ON_COMMAND(ID_TOOLS_AIML_REINDEX, OnAimlReindex)
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_DESTROY()
	ON_COMMAND(ID_TOOLS_CONTEXT_DIALOG, OnDialog)
	ON_COMMAND(ID_TOOLS_CONTEXT_TARGETS, OnTarget)
	ON_COMMAND(ID_TOOLS_CONTEXT_COMMAND, OnCommand)
	ON_COMMAND(ID_TOOLS_KNOWLEDGEBASE_CASES, OnCases)
	ON_COMMAND(ID_TOOLS_KNOWLEDGEBASE_FACTS, OnFacts)
	ON_COMMAND(ID_TOOLS_KNOWLEDGEBASE_SUBJECTS, OnSubjects)
	ON_COMMAND(ID_TOOLS_KNOWLEDGEBASE_TREE, OnTree)
	ON_COMMAND(ID_TOOLS_PROCESS_FRAME, OnFrame)
	ON_COMMAND(ID_TOOLS_PROCESS_SCRIPTS, OnScripts)
	ON_COMMAND(ID_EDIT_GOTOLINE, OnEditGotoline)
	ON_COMMAND(ID_TOOLS_AIML_EDIT_TARGETS, OnToolsAimlEditTargets)
	ON_COMMAND(ID_TOOLS_AIML_QUICK_CATEGORY, OnToolsAimlQuickCategory)
	ON_COMMAND(ID_TOOLS_AIML_CLASSIFY, OnToolsAimlClassify)
	ON_COMMAND(ID_TOOLS_AIML_LIST, OnToolsAimlList)
	ON_COMMAND(ID_TOOLS_CONTEXT_SETGOSSIP, OnToolsContextSetgossip)
	ON_COMMAND(ID_TOOLS_CONTEXT_SETTARGET, OnToolsContextSettarget)
	ON_COMMAND(ID_TOOLS_AIML_LOADTARGETS, OnToolsAimlLoadtargets)
	ON_COMMAND(ID_TOOLS_AIML_SEARCHCATEGORIES, OnToolsAimlSearchCategories)
	ON_COMMAND(ID_TOOLS_CONTEXT_USE_MSAGENT, OnViewUseMsagent)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_CONTEXT_USE_MSAGENT, OnUpdateToolsContextUseMsagent)
	ON_COMMAND(ID_ATTRIBUTES, OnAttributes)
	ON_COMMAND(ID_RULES, OnRules)
	ON_COMMAND(ID_TOOLS_USEGRAPHMASTER, OnToolsUsegraphmaster)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView construction/destruction

CAIMLpadView::CAIMLpadView()
{
	m_pLearnDialog = new CLearnDlg;
	m_pCmdDialog = new CCmdDlg;

	m_pSubjectDialog = new CSubjectsDlg;
	m_pCaseDialog = new CCasesDlg;
	m_pFactDialog = new CFactsDlg;
	m_pTreeDialog = new CTreeDlg;

	m_pScriptDialog = new CScriptDlg;
	m_pFrameDialog = new CFrameDlg;

	m_pAttrDialog = new CAttrDlg;
	m_pRuleDialog = new CRuleDlg;

	m_pGotoDialog = new CGotoDlg;

	m_pDumpDialog = new CDumpDlg;
	m_pListDialog = new CListDlg;
	m_pSearchDialog = new CSearchDlg;
	m_pTargetDialog = new CTargetDlg;

	blnGraphmaster = false;
	blnDialog = false;
	blnTarget = false;
	blnThinking = false;

	// SZ: MSAgent flags
	blnUseMSAgent = false;
	blnMSAgent = false;			// Are we using the MSAgent?
	blnMSAgentActive = false;	// Is the MSAgent activated?

	pu.host = this;
	pu.bot = &bot;

	bot.pu = &pu;
	bot.initialize();

	m_pSubjectDialog->kb = &(pu.kb);
	m_pCaseDialog->kb = &(pu.kb);
	m_pFactDialog->kb = &(pu.kb);
	m_pTreeDialog->kb = &(pu.kb);

	m_pScriptDialog->pgm = &(pu.pgm);
	m_pFrameDialog->pgm = &(pu.pgm);

	m_pAttrDialog->atr = &(pu.atr);
	m_pAttrDialog->xpt = &(pu.xpt);
	m_pRuleDialog->xpt = &(pu.xpt);

	m_pScriptDialog->pu = &pu;
	m_pSubjectDialog->pu = &pu;
	m_pCaseDialog->pu = &pu;
	m_pListDialog->pu = &pu;
	m_pSearchDialog->pu = &pu;
	m_pTargetDialog->pu = &pu;

	m_pDumpDialog->bot = &bot;
	m_pListDialog->bot = &bot;
	m_pSearchDialog->bot = &bot;
	m_pTargetDialog->bot = &bot;

	atpos = topos = 0;

	if (!bot.dbfnd) {
		CString str;
		str.Format(TEXT("AIML database file %hs is missing."),bot.fnidx);
		MessageBox(str,TEXT("AIML Database Error"),MB_OK);
	}

	if (bot.intAgent > 0) blnUseMSAgent = StartMSAgent(bot.fnagnt);
	if (blnUseMSAgent) SetMSAgentTTS(bot.fnagntTTS);
}

CAIMLpadView::~CAIMLpadView()
{
	delete m_pLearnDialog;
	delete m_pCmdDialog;
	delete m_pSubjectDialog;
	delete m_pCaseDialog;
	delete m_pFactDialog;
	delete m_pTreeDialog;
	delete m_pScriptDialog;
	delete m_pFrameDialog;
	delete m_pAttrDialog;
	delete m_pRuleDialog;
	delete m_pGotoDialog;
	delete m_pDumpDialog;	
	delete m_pListDialog;	
	delete m_pSearchDialog;
	delete m_pTargetDialog;	
}

BOOL CAIMLpadView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	BOOL bPreCreated = CEditView::PreCreateWindow(cs);
	cs.style &= ~(ES_AUTOHSCROLL|WS_HSCROLL);	// Enable word-wrapping

	return bPreCreated;
}

void CAIMLpadView::OnInitialUpdate() 
{
	int i;
	CEditView::OnInitialUpdate();

	if (blnUseMSAgent && bot.intAgent > 1) OnViewUseMsagent();
	if (bot.dbfnd) if (pu.startbot) OnDialog();
	if (blnGraphmaster != bot.usegraphmaster) OnToolsUsegraphmaster();
	if (bot.ircactive)
	{
	if (serv>0) 	i = SetTimer(2,1000,0);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView drawing

void CAIMLpadView::OnDraw(CDC* pDC)
{
	CAIMLpadDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView printing

BOOL CAIMLpadView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default CEditView preparation
	return CEditView::OnPreparePrinting(pInfo);
}

void CAIMLpadView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Default CEditView begin printing.
	CEditView::OnBeginPrinting(pDC, pInfo);
}

void CAIMLpadView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Default CEditView end printing
	CEditView::OnEndPrinting(pDC, pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView diagnostics

#ifdef _DEBUG
void CAIMLpadView::AssertValid() const
{
	CEditView::AssertValid();
}

void CAIMLpadView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CAIMLpadDoc* CAIMLpadView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAIMLpadDoc)));
	return (CAIMLpadDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CAIMLpadView message handlers

void CAIMLpadView::OnTimer(UINT nIDEvent) 
{

	if (nIDEvent==1)
	{
	char req[2048];
	char reply[4096];
	CString display;
	int nAt, pos;
	CEdit& edit = GetEditCtrl();
	DWORD dwSel = edit.GetSel();
	atpos = HIWORD(dwSel);
	topos = LOWORD(dwSel);

	strcpy(req,bot.second);

	if (bot.newsec && *req) {
		nAt = edit.LineFromChar();

		*reply = 0;
		bot.reevaluate(req,reply);
		strcat(reply,"\r\n");

		if (blnMSAgent) SpeakMSAgent(reply);
		
		strcpy(bot.second,"");
		bot.newsec = false;

		display = reply;
		if (atline > nAt) {
			pos = edit.LineIndex(nAt);
		} else {
			pos = edit.LineIndex(atline);
		}
		edit.SetSel(pos,pos);

		edit.ReplaceSel(display); 

		atpos = atpos + strlen(reply);
		edit.SetSel(atpos,atpos);

		dwSel = edit.GetSel();
		atpos = HIWORD(dwSel);
		topos = LOWORD(dwSel);
		atline = edit.LineFromChar();
	}

	CEditView::OnTimer(nIDEvent);
	KillTimer(1);
	}

	if (nIDEvent==2)
	{
		if (bot.ircactive )
		{
		KillTimer(2);
		bot.runIrc();
		SetTimer(2,200,0);
		}
	}

}

void CAIMLpadView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CEditView::OnKeyDown(nChar, nRepCnt, nFlags);

	if(nChar == VK_DELETE)
	{
		CEdit& edit = GetEditCtrl();
		TEXTMETRIC tm;
		CDC* pDC = edit.GetDC();
		pDC->GetTextMetrics(&tm);
		edit.ReleaseDC(pDC);
		CRect r;
		edit.GetRect(&r);
		int noOfVisibleLines = r.Height() / tm.tmHeight;

		if(edit.GetLineCount() <= noOfVisibleLines)
		{
			long lwStyle = ::GetWindowLong(edit.GetSafeHwnd(), GWL_STYLE);
			if(!(lwStyle & WS_VSCROLL))
				return;
			int nFirstVisible = edit.GetFirstVisibleLine();
		    edit.LineScroll(-nFirstVisible, 0);

			lwStyle &= ~WS_VSCROLL;
			::SetWindowLong(edit.GetSafeHwnd(), GWL_STYLE, lwStyle);
		}
	}
}

void CAIMLpadView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	char req[2048];
	char reply[4096];
	CString display;
	TCHAR inp[200];
	CEdit& edit = GetEditCtrl();
	int nAt, nTo;
	int i, j, k;
	int x, y, z;
	int iRet;
	UINT nElapse;
	DWORD dwSel;

	if (nChar == VK_RETURN) {
		if (pu.blnInput) {
			dwSel = edit.GetSel();
			nAt = HIWORD(pu.dwAtSel);
			nTo = HIWORD(dwSel);
			i = edit.LineFromChar(nAt);
			j = edit.LineFromChar(nTo);
			for (k = i,z = 0;k <= j;k++) {
				y = edit.GetLine(k,inp,200); inp[y] = 0;
				x = edit.LineIndex(k);
				for (y = nAt - x;inp[y] && nAt < nTo && z < 4094;y++,nAt++) 
					if (inp[y] >= 32 && inp[y] <= 127) reply[z++] = (char) inp[y];
			}
			reply[z] = 0;
			CEditView::OnChar(nChar, nRepCnt, nFlags);
			if (!blnDialog) SetCaretBlinkTime(500);
			UpdateWindow();
			if (pu.resume(reply)) theApp.GetMainWnd()->PostMessage(WM_COMMAND,ID_APP_EXIT,0);
			if (blnGraphmaster != bot.usegraphmaster) OnToolsUsegraphmaster();
			if (blnDialog) {
				if (bot.stopbot) OnDialog();
				else {
					Sleep(10);
					dwSel = edit.GetSel();
					atpos = HIWORD(dwSel);
					topos = LOWORD(dwSel);
					atline = edit.LineFromChar();
				}
			}
			return;
		} else if (blnDialog) {
			nAt = edit.LineFromChar();
			if (atline > nAt) atline = nAt;

			for (k = 0,i = atline;i <= nAt;i++) {
				y = edit.GetLine(i,inp,200); inp[y] = 0;
				for (j = 0;inp[j];j++) {
					if (inp[j] >= 32 && inp[j] <= 127) req[k++] = (char) inp[j];
					inp[j] = 0;
				}
			}
			req[k] = 0;

			if (blnThinking) { UpdateWindow(); return; }

			blnThinking = true;
			strcpy(reply,"");
			GetParent()->BeginWaitCursor();
			iRet = bot.getAnswer(req,reply);

			if (blnMSAgent) SpeakMSAgent(reply);

			strcat(reply,"\r\n");
			GetParent()->EndWaitCursor();
			blnThinking = false;
			
			if (pu.blnInput) return;
			if (bot.stopbot) OnDialog();
			display = reply;
			edit.ReplaceSel(TEXT("\r\n"));
			edit.ReplaceSel(display); 
			Sleep(10);
			dwSel = edit.GetSel();
			atpos = HIWORD(dwSel);
			topos = LOWORD(dwSel);
			atline = edit.LineFromChar();
			if (bot.clrscrn) {
				OnEditSelectAll();
				OnEditClear();
				bot.clrscrn = false;
			}
 			if (bot.newsec && !iRet) { 
				nElapse = 2000 + random(8000);
				i = SetTimer(1,nElapse,0);
			}
			if (iRet) {
				if (bot.nosave) edit.SetModify(false);
				theApp.GetMainWnd()->PostMessage(WM_COMMAND,ID_APP_EXIT,0);
				return;
			}
			nChar = 0;
		} 
	}
	CEditView::OnChar(nChar, nRepCnt, nFlags);
	UpdateWindow();
}

void CAIMLpadView::OnDestroy() 
{
	bot.unloadgm();
	SetCaretBlinkTime(500);
	CEditView::OnDestroy();
}

void CAIMLpadView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	CEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
	
	if (*(pu.startup)) {
		pu.onCmd(pu.startup);
		delete pu.startup;
		pu.startup = new char[2];
		*(pu.startup) = 0;
		if (blnGraphmaster != bot.usegraphmaster) OnToolsUsegraphmaster();
	}
}

void CAIMLpadView::OnDialog() 
{
	CString str;
	char *req;
	char *reply;
	int iRet;
	int i;
	UINT nElapse;
	DWORD dwSel;

	if (pu.blnInput) return;

	if (!bot.dbfnd) {
		str.Format(TEXT("AIML database file %hs is missing.  Bot chat is not available.  Some script commands may abort program.  See the System Documentation for proper setup configuration."),bot.fnidx);
		MessageBox(str,TEXT("AIML Database Error"),MB_OK);
		return;
	}

	bot.stopbot = false;

	CEdit& edit = GetEditCtrl();
	TCHAR inp[200];

	if (bot.useconnect) {
		GetParent()->BeginWaitCursor();
		req = new char[32];
		reply = new char[4096];
		strcpy(req,"CONNECT");
		strcpy(reply,"");
		iRet = bot.getAnswer(req,reply);

		if (blnMSAgent) SpeakMSAgent(reply);

		if (*reply && !pu.blnInput) {
			strcat(reply,"\r\n");
			str = reply;
			edit.ReplaceSel(str); 
			Sleep(10);
			dwSel = edit.GetSel();
			atpos = HIWORD(dwSel);
			topos = LOWORD(dwSel);
			atline = edit.LineFromChar();
			if (bot.clrscrn) {
				OnEditSelectAll();
				OnEditClear();
				bot.clrscrn = false;
			}
			if (bot.newsec && !iRet) { 
				nElapse = 2000 + random(8000);
				i = SetTimer(1,nElapse,0);
			}
		}
		delete req;
		delete reply;
		bot.useconnect = false;
		GetParent()->EndWaitCursor();
		if (iRet) {
			theApp.GetMainWnd()->PostMessage(WM_COMMAND,ID_APP_EXIT,0);
			return;
		}
	}
	
	CMenu* menu_bar = theApp.GetMainWnd()->GetMenu();
	CMenu* tool_menu = menu_bar->GetSubMenu(6);
	CToolBarCtrl* pBar = (CToolBarCtrl*) GetParentFrame()->GetControlBar(AFX_IDW_TOOLBAR);

	if (blnDialog = !blnDialog) {
		tool_menu->CheckMenuItem(ID_TOOLS_CONTEXT_DIALOG, MF_CHECKED | MF_BYCOMMAND);
		pBar->CheckButton(ID_TOOLS_CONTEXT_DIALOG, true);
		SetCaretBlinkTime(200);
		
		if (blnMSAgent && !blnMSAgentActive) {	// SZ: If we want MSAgent & is not activated...
			if (ShowMSAgent() == SUCCESS) blnMSAgentActive = true; // SZ:Show the character...
		}

		DWORD dwSel = edit.GetSel();
		atpos = HIWORD(dwSel);
		topos = LOWORD(dwSel);
		atline = edit.LineFromChar();
		edit.GetLine(atline,inp,200);
	} else {
		tool_menu->CheckMenuItem(ID_TOOLS_CONTEXT_DIALOG, MF_UNCHECKED | MF_BYCOMMAND);
		pBar->CheckButton(ID_TOOLS_CONTEXT_DIALOG, false);
		SetCaretBlinkTime(500);

		if (blnMSAgent && blnMSAgentActive) {	// SZ: If MSAgent is activated then...
			if (HideMSAgent() == SUCCESS) blnMSAgentActive = false;	// SZ: Hide the MSAgent.
		}
	}
}

void CAIMLpadView::OnTarget() 
{
	CMenu* menu_bar = theApp.GetMainWnd()->GetMenu();
	CMenu* tool_menu = menu_bar->GetSubMenu(6); 

	if (blnTarget = !blnTarget) {
		tool_menu->CheckMenuItem(ID_TOOLS_CONTEXT_TARGETS, MF_CHECKED | MF_BYCOMMAND);
	} else {
		tool_menu->CheckMenuItem(ID_TOOLS_CONTEXT_TARGETS, MF_UNCHECKED | MF_BYCOMMAND);
	}
	bot.logtarget = blnTarget;
}

void CAIMLpadView::OnCommand() 
{
	CEdit& edit = GetEditCtrl();
	DWORD dwSel = edit.GetSel();
	CString s;

	s.Format(_T("%s.%s"),pu.kb.curSub->m_name,pu.kb.curSub->m_cases.curCase->m_name);
	m_pCmdDialog->m_status = s;
	if(m_pCmdDialog->DoModal() != IDOK)	return;
	if (pu.onCmd(m_pCmdDialog->m_command)) theApp.GetMainWnd()->PostMessage(WM_COMMAND,ID_APP_EXIT,0);
	if (blnGraphmaster != bot.usegraphmaster) OnToolsUsegraphmaster();
	if (blnDialog) {
		if (bot.stopbot) OnDialog();
		else {
			Sleep(10);
			dwSel = edit.GetSel();
			atpos = HIWORD(dwSel);
			topos = LOWORD(dwSel);
			atline = edit.LineFromChar();
		}
	}
}

void CAIMLpadView::OnSubjects() 
{
	m_pSubjectDialog->m_subject = pu.kb.curSub->m_name;

	m_pSubjectDialog->DoModal();
}

void CAIMLpadView::OnCases() 
{
	m_pCaseDialog->m_subject = pu.kb.curSub->m_name;
	m_pCaseDialog->m_case = pu.kb.curSub->m_cases.curCase->m_name;
	
	m_pCaseDialog->DoModal();
}

void CAIMLpadView::OnFacts() 
{
	CString str;

	str.Format(_T("%s.%s"),pu.kb.curSub->m_name,pu.kb.curSub->m_cases.curCase->m_name);

	m_pFactDialog->m_case = str;
	m_pFactDialog->m_fact = _T("");
	
	m_pFactDialog->DoModal();
}

void CAIMLpadView::OnTree() 
{
	m_pTreeDialog->DoModal();
}

void CAIMLpadView::OnScripts() 
{
	m_pScriptDialog->DoModal();
}

void CAIMLpadView::OnFrame() 
{
	m_pFrameDialog->m_name = _T("");
	m_pFrameDialog->DoModal();
}

void CAIMLpadView::OnEditGotoline() 
{
	if(m_pGotoDialog->DoModal() != IDOK) return;

	if(m_pGotoDialog->m_nLineNum < 1) {	MessageBox(_T("Invalid Line Number")); return; }

	CEdit &edit = GetEditCtrl();
	int i = edit.LineFromChar();            // this is the current line the cursor is on
	int nLine = m_pGotoDialog->m_nLineNum;  // line number to go to

	if(nLine > (edit.GetLineCount())) { MessageBox(_T("Number too big")); return; }

	// move window and caret
	--nLine;                      // edit control is zero based
	edit.LineScroll(nLine - i);   // new line number - the current line

	int idx;
	idx = edit.LineIndex(nLine);
	edit.SetSel(idx, idx);
}

void CAIMLpadView::OnAimlLearn() 
{
	if(m_pLearnDialog->DoModal() == IDOK) { bot.loadchunks(); bot.reloadgm(); }
}

void CAIMLpadView::OnAimlSort() 
{
	if (blnThinking) return;
	blnThinking = true;
	GetParent()->BeginWaitCursor();
	m_pLearnDialog->sortforstandardaiml();	
	GetParent()->EndWaitCursor();
	blnThinking = false;
}

void CAIMLpadView::OnAimlRemoveDups() 
{
	if (blnThinking) return;
	blnThinking = true;
	GetParent()->BeginWaitCursor();
	m_pLearnDialog->mergecategories();	
	GetParent()->EndWaitCursor();
	blnThinking = false;
	bot.reloadgm();
}

void CAIMLpadView::OnAimlDump() 
{
	m_pDumpDialog->DoModal();
}

void CAIMLpadView::OnAimlReindex() 
{
	if (blnThinking) return;
	blnThinking = true;
	GetParent()->BeginWaitCursor();
	m_pLearnDialog->reindex();	
	GetParent()->EndWaitCursor();
	blnThinking = false;
}

void CAIMLpadView::OnToolsAimlList() 
{
	m_pListDialog->DoModal();
}

void CAIMLpadView::OnToolsAimlSearchCategories() 
{
	m_pSearchDialog->DoModal();
}

void CAIMLpadView::OnToolsAimlEditTargets() 
{
	m_pTargetDialog->DoModal();
}

void CAIMLpadView::OnToolsAimlClassify() 
{
	char fname[MAX_PATH_LEN];

	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Open text dialog file to classify into AIML categories...");

	if (dlg.DoModal() == IDCANCEL)	return;

	GetParent()->BeginWaitCursor();
	sprintf(fname,"%s",dlg.GetPathName());
	delete dlg;

	pu.classifyaiml(fname);
	GetParent()->EndWaitCursor();
}

void CAIMLpadView::OnToolsAimlQuickCategory() 
{
	char fname[MAX_PATH_LEN];

	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Open text dialog file to convert to AIML categories...");

	if (dlg.DoModal() == IDCANCEL)	return;

	GetParent()->BeginWaitCursor();
	sprintf(fname,"%s",dlg.GetPathName());
	delete dlg;

	pu.convertaiml(fname);
	GetParent()->EndWaitCursor();
}

void CAIMLpadView::OnToolsAimlLoadtargets() 
{
	char fname[MAX_PATH_LEN];

	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Open text dialog file to convert to targets...");

	if (dlg.DoModal() == IDCANCEL)	return;

	GetParent()->BeginWaitCursor();
	sprintf(fname,"%s",dlg.GetPathName());
	delete dlg;

	pu.converttargets(fname);
	GetParent()->EndWaitCursor();
}

void CAIMLpadView::OnToolsContextSettarget() 
{
	CString szFilter = "Text Files (*.txt)|*.txt|XML Files (*.xml)|*.xml|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Set Targets Log File As...");

	if (dlg.DoModal() == IDCANCEL)	return;

	sprintf(bot.fntar,"%s",dlg.GetPathName());
	delete dlg;
}

void CAIMLpadView::OnToolsContextSetgossip() 
{
	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Set Gossip File As...");

	if (dlg.DoModal() == IDCANCEL)	return;

	sprintf(bot.fngos,"%s",dlg.GetPathName());
	delete dlg;
}

void CAIMLpadView::OnAttributes() 
{
	m_pAttrDialog->DoModal();
}

void CAIMLpadView::OnRules() 
{
	m_pRuleDialog->DoModal();
}

void CAIMLpadView::OnToolsUsegraphmaster() 
{
	CMenu* menu_bar = theApp.GetMainWnd()->GetMenu();
	CMenu* tool_menu = menu_bar->GetSubMenu(6); 

	if (blnGraphmaster = !blnGraphmaster) {
		tool_menu->CheckMenuItem(ID_TOOLS_USEGRAPHMASTER, MF_CHECKED | MF_BYCOMMAND);
	} else {
		tool_menu->CheckMenuItem(ID_TOOLS_USEGRAPHMASTER, MF_UNCHECKED | MF_BYCOMMAND);
	}
	bot.usegraphmaster = blnGraphmaster;
}

// SZ: Added for use with Microsoft Agent and AIMLpad.
void CAIMLpadView::OnViewUseMsagent() 
{
	CMenu* menu_bar = theApp.GetMainWnd()->GetMenu();
	CMenu* view_menu = menu_bar->GetSubMenu(2);

	if (blnMSAgent = !blnMSAgent) {	// Checkmark the menu item.
		view_menu->CheckMenuItem(ID_TOOLS_CONTEXT_USE_MSAGENT, MF_CHECKED | MF_BYCOMMAND);

		if (blnDialog && !blnMSAgentActive) {	// If we are in Dialog mode & MSAgent is NOT activated...
			if (ShowMSAgent() == SUCCESS) blnMSAgentActive = true;	// Show the character,
		}
	} else {						// Uncheckmark the menu item.
		view_menu->CheckMenuItem(ID_TOOLS_CONTEXT_USE_MSAGENT, MF_UNCHECKED | MF_BYCOMMAND);

		if (blnDialog && blnMSAgentActive) {	// If we are in Dialog mode & MSAgent is activated...
			if (HideMSAgent() == SUCCESS) blnMSAgentActive = false;	// Hide the character,
		}	
	}
}

void CAIMLpadView::OnUpdateToolsContextUseMsagent(CCmdUI* pCmdUI) 
{
	if (blnUseMSAgent == false) pCmdUI->Enable(false);
}
