/* Copyright (c) 2000-2002, Jacco Bikker
 * Copyright (c) 2002-2003, Gary Dubuque
 *
 * This file is part of NP+AIML.
 * NP+AIML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * NP+AIML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with NP+AIML; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

// LearnDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AIMLpad.h"
#include "LearnDlg.h"

#include "sys/types.h"
#include "time.h"

#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "support.h"
#include "io.h"

extern CAIMLpadApp theApp;

#define SORT_ITEMS 250
#define MAX_PASSES 25

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLearnDlg dialog


CLearnDlg::CLearnDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLearnDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLearnDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	blnThinking = false;
	topic = new char[MAX_PHRASE_LEN];
}


CLearnDlg::~CLearnDlg()
{
	delete topic;
}


BOOL CLearnDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	GetDlgItem(IDC_LEARN_FN)->SetWindowText(TEXT("C:\\Temp\\AIMLPAD\\Debug\\AIML\\testcase1.txt"));
	GetDlgItem(IDC_CHECKLIST)->SendMessage(BM_SETCHECK, BST_CHECKED, NULL);
	GetDlgItem(IDC_CHECK_MERGE)->SendMessage(BM_SETCHECK, BST_UNCHECKED, NULL);

	return TRUE;
}

void CLearnDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLearnDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLearnDlg, CDialog)
	//{{AFX_MSG_MAP(CLearnDlg)
	ON_BN_CLICKED(IDC_LEARN_BROWSE, OnLearnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLearnDlg message handlers

void CLearnDlg::OnOK() 
{
	CStatusBar* status_bar = (CStatusBar *)theApp.GetMainWnd()->GetDlgItem(AFX_IDW_STATUS_BAR);
	CString fn;
	TCHAR szInput[MAX_PHRASE_LEN];
	char szPath[MAX_PATH_LEN];
	char szDir[MAX_PATH_LEN];
	char szName[MAX_PATH_LEN];
	char szNext[MAX_PATH_LEN];
	int iCheckList;
	int iMerge;
	int iRet = 0;
	int i;

	if (blnThinking) return;
	blnThinking = TRUE;
	GetParent()->BeginWaitCursor();

	GetDlgItem(IDC_LEARN_FN)->GetWindowText(fn);
	sprintf(szPath,"%s",fn);
	if (*szPath) {
		strcpy(szDir,szPath);
		for (i = strlen(szDir) - 1;i >= 0 && szDir[i] != '\\';i--);
		szDir[i] = '\0';
		if (szPath[i] == '\\') i++;
		strcpy(szName,szPath + i);

		iCheckList = IsDlgButtonChecked(IDC_CHECKLIST);
		iMerge = IsDlgButtonChecked(IDC_CHECK_MERGE);

		iRet = prepareaiml(szDir,szName,szNext,iMerge,iCheckList);
		if (iRet) {
			wsprintf(szInput,TEXT("Not Found:%hs"),szName);
			status_bar->SetWindowText(szInput);
			UpdateWindow();
		} else {
			wsprintf(szInput,TEXT("loading %hs..."),szNext);
			status_bar->SetWindowText(szInput);
			while (foreachaiml(szDir,szName,szNext,iCheckList)) {
				wsprintf(szInput,TEXT("loading %hs..."),szNext);
				status_bar->SetWindowText(szInput);
				UpdateWindow();
			}
			if (*szNext) {
				status_bar->SetWindowText(TEXT("Sorting patterns..."));
				UpdateWindow();
				sortaiml();
				status_bar->SetWindowText(TEXT("Cleanup..."));
				UpdateWindow();
				iRet = cleanupaiml();
				status_bar->SetWindowText(TEXT("Finished Learning AIML."));
			} else {
				wsprintf(szInput,TEXT("%hs"),szName);
				status_bar->SetWindowText(szInput);
				UpdateWindow();
				iRet = 1;
			}
			MessageBeep(MB_ICONEXCLAMATION);
		}
	} else {
		status_bar->SetWindowText(TEXT("Sorting patterns..."));
		UpdateWindow();
		sortaiml();
		status_bar->SetWindowText(TEXT("Cleanup..."));
		UpdateWindow();
		iRet = cleanupaiml();
		MessageBeep(MB_ICONEXCLAMATION);
		status_bar->SetWindowText(TEXT("Finished Learning AIML."));
	}

	GetParent()->EndWaitCursor();
	blnThinking = FALSE;
	
	if (!iRet) CDialog::OnOK();
}

void CLearnDlg::OnLearnBrowse() 
{
	CString szFilter = "Text Files (*.txt)|*.txt|AIML Files (*.aiml)|*.aiml|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	if (dlg.DoModal() == IDCANCEL)	return;

	GetDlgItem(IDC_LEARN_FN)->SetWindowText(dlg.GetPathName());	
}

void CLearnDlg::replacer(char *line)
{
	if (!*line) return;
	char *pos;
	char *buf = new char[MAX_BUF_LEN];

	while (strstr(line,"<?xml")) remove(line,"<?xml","?>");
	while (strstr(line,"<!--")) remove(line,"<!--","-->");
	while (strstr(line,"<meta")) remove(line,"<meta","/>");

	if (strstr(line,"<aiml version=\"1.0.1\">")) { convertversion = 0; replace(line,"<aiml version=\"1.0.1\">",""); }
	if (strstr(line,"<aiml version=")) remove(line,"<aiml version=",">");
	replace(line,"<aiml>","");
	replace(line,"</aiml>","");
	replace(line,"<category>","");
	replace(line,"</category>","");

	if (convertversion) {
		if (strstr(line,"<alice>")) { convertversion = 2; replace(line,"<alice>",""); }
		replace(line,"</alice>","");
		while (replace(line,"<getversion/>","<version/>"));
		while (replace(line,"<bot name=\"name\"/>","<bot/>"));
		while (replace(line,"<botname/>","<bot/>"));
		while (replace(line,"<name/>","<bot/>"));
		while (replace(line,"<getvar ","<get "));
		while (replace(line,"<setvar ","<set "));
		while (replace(line,"</setvar>","</set>"));
		while (replace(line,"<they/>","<get name=\"they\"/>"));
		while (replace(line,"<getsize/>","<size/>"));
		while (replace(line,"<getname/>","<get name=\"name\"/>"));
		while (replace(line,"<gettopic/>","<get name=\"topic\"/>"));
		while (replace(line,"<setname>","<set name=\"name\">"));
		while (replace(line,"</setname>","</set>"));
		while (replace(line,"<set_male/>","<set name=\"gender\">male</set>"));
		while (replace(line,"<set_female/>","<set name=\"gender\">female</set>"));
		while (replace(line,"<settopic/>","<set name=\"topic\"></set>"));
		while (replace(line,"<settopic>","<set name=\"topic\">"));
		while (replace(line,"</settopic>","</set>"));
		while (replace(line,"<get_ip/>","<id/>"));
		while ((pos = strstr(line,"<get_"))) {
			strcpy(buf,pos + 5);
			strcpy(pos,"<get name=\"");
			strcat(pos,buf);
			pos = strstr(pos,"/>");
			strcpy(buf,pos);
			strcpy(pos,"\"");
			strcat(pos,buf);
		}
		while ((pos = strstr(line,"<set_"))) {
			strcpy(buf,pos + 5);
			strcpy(pos,"<set name=\"");
			strcat(pos,buf);
			pos = strstr(pos,">");
			strcpy(buf,pos);
			strcpy(pos,"\"");
			strcat(pos,buf);
		}
		while ((pos = strstr(line,"</set_"))) {
			strcpy(buf,pos);
			strcpy(pos,"</set");
			pos = strstr(buf,">");
			strcat(line,pos);
		}
	}

	if ((pos = strstr(line,"<topic name=\""))) {
		XMLtag(pos,buf);
		XMLvalue(buf,"name",topic);
		replace(line,buf,"");
	}
	if (strstr(line,"</topic>")) { *topic = 0; }
	replace(line,"</topic>","");

	while(replace(line,"\r",""));
	while(replace(line,"\n",""));

	delete buf;
}  

int CLearnDlg::checkline(char *line)
{
	cleanspace(line);
	if (!*line) return 0;

	if (strstr(line,"<alice") || strstr(line,"<aiml")) {
		if (cc != 0) return 1;
		cc += 1;
	}
	if (strstr(line,"<topic ")) {
		if (cc != 1) return 1;
		cc += 2;
	}
	if (strstr(line,"<category>")) {
		if (cc != 1 && cc != 3) return 1;
		cc += 4;
	}
	if (strstr(line,"<pattern>")) {
		if (!(cc & 1) || !(cc & 4)) return 1;
		cc += 8;
	}
	if (strstr(line,"</pattern>")) {
		if (!(cc & 1) || !(cc & 4) || hh || he || !(cc & 8)) return 1;
		cc -= 8;
		hp = 1;
	}
	if (strstr(line,"<that>")) {
		if (!(cc & 1) || !(cc & 4) || !(hp) || he) return 1;
		cc += 16;
	}
	if (strstr(line,"</that>")) {
		if (!(cc & 1) || !(cc & 4) || !(hp) || he || !(cc & 16)) return 1;
		cc -= 16;
		hh = 1;
	}
	if (strstr(line,"<template>")) {
		if (!(cc & 1) || !(cc & 4) || !(hp)) return 1;
		cc += 32;
	}
	if (strstr(line,"</template>")) {
		if (!(cc & 1) || !(cc & 4) || !(hp) || !(cc & 32)) return 1;
		cc -= 32;
		he = 1;
	}
	if (strstr(line,"</category>")) {
		if (!(cc & 1) || !(cc & 4) || !(hp) || !(he)) return 1;
		cc -= 4;
		hp = 0;
		hh = 0;
		he = 0;
	}
	if (strstr(line,"</topic>")) {
		if (cc != 3) return 1;
		cc -= 2;
	}
	if (strstr(line,"</alice>") || strstr(line,"</aiml>")) {
//??    if (cc != 1) return 1;
		cc -= 1;
	}
	return 0;
}

long CLearnDlg::splitup(char *src)
{
	long patterns = 0;
	long templates = 0;
	long linenr = 1;
	fpos_t tpos;
	char *at;
	int iRet = 0;

	cc = 0;
	hp = 0;
	hh = 0;
	he = 0;
	convertversion = 1;
	*topic = 0;

	FILE *aiml = fopen(src,"r");
	if (!aiml) { return (-1); }

	char *line = new char[MAX_BUF_LEN];

	fgets(line,MAX_BUF_LEN,aiml);
	if (checkline(line)) {
		delete line;
		fclose(aiml);
		return 1;
	}

	char *buf = new char[MAX_BUF_LEN];
	char *fnpat = new char[MAX_PATH_LEN];
	char *fntmp = new char[MAX_PATH_LEN];

	strcpy(fnpat,fullpath("db\\unsorted.txt"));
	strcpy(fntmp,fullpath("db\\templates.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_BUF_LEN,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"unsorted=",9))   strcpy(fnpat,buf + 9);
			if (!strncmp(buf,"templates=",10)) strcpy(fntmp,buf + 10);
		}
		fclose(fdirs);
	}
	FILE *ftemp = fopen(fntmp,"a");
	FILE *fpatt = fopen(fnpat,"a");

	delete fntmp;
	delete fnpat;

	fprintf(ftemp,"==%s==\n",src);
	if (fgetpos(ftemp,&tpos)) { delete buf; delete line; fclose(aiml); fclose(ftemp); fclose(fpatt); return -3; }

	*buf = 0;
	char *next = new char[MAX_BUF_LEN];

	while (!feof(aiml)) {
		*next = 0;
		if (!*line || strncmp(line,"<template>",10)) {
			if (!strstr(line,"<pattern>")) { fgets(next,MAX_BUF_LEN,aiml); linenr++; }
			if (checkline(next)) { iRet = linenr; break;}
			if (strstr(line,"<!--")) {
				while (!strstr(line,"-->")) {
					if (strlen(line) + strlen(next) > MAX_BUF_LEN) { iRet = linenr; break; }
					strcat(line,next);
					linenr++;
					fgets(next,MAX_BUF_LEN,aiml);
					if (checkline(next)) { iRet = linenr; break;}
				}
				if (iRet) break;
			}
			replacer(line);
			if (strstr(line,"<pattern>") && (strstr(line,"<that>")) || strstr(next,"<that>")) {
				while (!strstr(line,"</that>")) {
					if (strlen(line) + strlen(next) > MAX_BUF_LEN) { iRet = linenr; break; }
					strcat(line,next);
					replacer(line);
					linenr++;
					fgets(next,MAX_BUF_LEN,aiml);
					if (checkline(next)) { iRet = linenr; break;}
				}
				if (iRet) break;
				if (strstr(line,"<pattern>")) patterns++;
				strcpy(buf,at = (strstr(line,"</that>") + 7));
				*(at - 7) = 0;
				replace(line,"<pattern>","");
				replace(line,"</pattern>","");
				replace(line,"<that>","[that]");
				fprintf(fpatt,"%s",line);
			} else if (strstr(line,"<pattern>")) {
				while (!strstr(line,"</pattern>")) {
					if (strlen(line) + strlen(next) > MAX_BUF_LEN) { iRet = linenr; break; }
					strcat(line,next);
					replacer(line);
					linenr++;
					fgets(next,MAX_BUF_LEN,aiml);
					if (checkline(next)) { iRet = linenr; break;}
				}
				if (iRet) break;
				patterns++;
				strcpy(buf,at = (strstr(line,"</pattern>") + 10));
				*at = 0;
				replace(line,"<pattern>","");
				replace(line,"</pattern>","");
				fprintf(fpatt,"%s",line);
			}
		} else replacer(line);
		if (strstr(line,"<template>")) {
			while (!strstr(line,"</template>")) {
				if (strlen(line) + strlen(next) > MAX_BUF_LEN) { iRet = linenr; break; }
				strcat(line,next);
				replacer(line);
				linenr++;
				fgets(next,MAX_BUF_LEN,aiml);
				if (checkline(next)) { iRet = linenr; break;}
			}
			if (iRet) break;
			fgetpos(ftemp,&tpos);
			templates++;
			if (templates != patterns) {
				return -1 * linenr; // printf( "Pattern/Template count mismatch\n"); 
			}
			strcpy(buf,at = (strstr(line,"</template>") + 11)); 
			*at = 0;
			replace(line,"<template>","");
			replace(line,"</template>","");
			compress(line);
			fprintf(ftemp, "%s\n",line);
			if (*topic)	fprintf(fpatt,"[topic]%s",topic);
			fprintf(fpatt,"@%li\n",tpos);
		}
		for (at = buf;*at && *at <= ' ';at++);
		strcpy(line,at);
		strcat(line,next);
	}

	delete line;
	delete next;
	delete buf;

	fclose(aiml);
	fclose(fpatt);
	fclose(ftemp);
  
	return iRet;
}

int CLearnDlg::prepareaiml(char *dir,char *fname,char *atfn,int iMerge,int iList)
{
	char *fn = new char[MAX_PATH_LEN];

	if (*dir == 0) strcpy(dir,fullpath(""));
	if (dir[strlen(dir) - 1] == '\\') dir[strlen(dir) - 1] = 0;

	sprintf(fn,"%s\\%s",dir,fname);
	if (!fexists(fn)) { delete fn; return 1; }

	if (iList) {
		FILE *fnlist = fopen(fn,"r");
		if (!feof(fnlist)) {
			fgets(fn,MAX_PATH_LEN,fnlist);
			rtrim(fn);
			strcpy(atfn,fn);
		}
		fclose(fnlist);
	} else {
		strcpy(atfn,fname);
	}

	char *fnsrt = new char[MAX_PATH_LEN];
	char *fnpat = new char[MAX_PATH_LEN];
	char *fntmp = new char[MAX_PATH_LEN];
	char *buf = new char[MAX_LINE_SIZE];

	strcpy(fnsrt,fullpath("db\\unsorted.txt"));
	strcpy(fnpat,fullpath("db\\patterns.txt"));
	strcpy(fntmp,fullpath("db\\templates.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	while (!feof(fdirs)) {
		fgets(buf,MAX_LINE_SIZE,fdirs);
		rtrim(buf);
		if (!strncmp(buf,"unsorted=",9))   strcpy(fnsrt,buf + 9);
		if (!strncmp(buf,"patterns=",9))   strcpy(fnpat,buf + 9);
		if (!strncmp(buf,"templates=",10)) strcpy(fntmp,buf + 10);
	}
	fclose(fdirs);
	*buf = 0;

	FILE *fsort = fopen(fnsrt,"w");
	if (iMerge) {
		FILE *fpatt = fopen(fnpat,"r");
		while (!feof(fpatt)) {
			*buf = 0;
			fgets(buf,MAX_LINE_SIZE,fpatt);
			if (*buf && strncmp(buf,"<eof>",5)) fprintf(fsort,"%s",buf);
		}
		fclose(fpatt);
		fclose(fsort);
	} else {
		fclose(fsort);
		FILE *ftemp = fopen(fntmp,"w");
		fclose(ftemp);
	}

	delete buf;
	delete fn;
	delete fnsrt;
	delete fnpat;
	delete fntmp;
	return 0;
}

int CLearnDlg::foreachaiml(char *dir,char *fname,char *atfn,int iList)
{
	long iRet = 0;
	char *fn = new char[MAX_PATH_LEN];	
	char *fnfnd = new char[MAX_PATH_LEN];
	char *fnnxt = new char[MAX_PATH_LEN];
	char *atfnd;	
	struct _finddata_t c_file;
    long hFile;
	bool fnd;
	char *pos;

	sprintf(fn,"%s\\%s",dir,atfn);
	if (pos = strstr(fn,"*")) {
		if ((hFile = _findfirst(fn,&c_file)) != -1L) strcpy(fnfnd,c_file.name); else strcpy(fn,"<eof>");
		_findclose(hFile);
		while (*pos != '\\') pos--;
		strcpy(pos + 1,fnfnd);
		pos = strstr(atfn,"*");
		while (pos >= atfn && *pos != '\\') pos--;
		strcpy(pos + 1,fnfnd);
	}
	iRet = splitup(fn);
	if (iRet > 0 || iRet < -1) {
		sprintf(fname,"Err(%d): %ld in %s",cc,iRet,fn);
		*atfn = 0;
		delete fn;
		return 0;
	}

	if (iList) {
		sprintf(fn,"%s\\%s",dir,fname);
		FILE *fnlist = fopen(fn,"r");
		while ((fnlist!=NULL) && !feof(fnlist)) {
			fgets(fn,MAX_PATH_LEN,fnlist);
			rtrim(fn);
			if (!strcmp(fn,"<eof>")) break;
			if (!strcmp(fn,atfn)) break;
			if (pos = strstr(fn,"*")) {
				if (!strstr(fn,"\\")) { if (!strstr(atfn,"\\")) break; }
				else if (!strncmp(fn,atfn,pos - fn)) break;
			}
		}
		if ((fnlist!=NULL) && !feof(fnlist)) fgets(fnnxt,MAX_PATH_LEN,fnlist); else strcpy(fnnxt,"<eof>");
		if (!strcmp(fn,atfn)) strcpy(fn,fnnxt);
		else {
			if ((hFile = _findfirst(fn,&c_file)) != -1L) {
				pos = strstr(fn,"*");
				while (pos >= fn && *pos != '\\') pos--;
				atfnd = atfn + (pos - fn) + 1;
				if (strcmp(c_file.name,atfnd)) {
					fnd = false;
					while (!_findnext(hFile,&c_file)) if (!strcmp(c_file.name,atfnd)) { fnd = true; break; }
				} else fnd = true;
				if (fnd) {
					if (!_findnext(hFile,&c_file)) {
						pos = strstr(fn,"*");
						while (pos >= fn && *pos != '\\') pos--;
						strcpy(pos + 1,c_file.name); 
					} else strcpy(fn,fnnxt);
				} else strcpy(fn,fnnxt);
			} else strcpy(fn,fnnxt);
			_findclose(hFile);
		}
		if (fnlist!=NULL) fclose(fnlist);
		rtrim(fn);
		strcpy(atfn,fn);
	} else strcpy(atfn,"<eof>");

	delete fn;
	delete fnfnd;
	delete fnnxt;
	return *atfn && strcmp(atfn,"<eof>");
}

void CLearnDlg::sortaiml()
{
	int i = 0;
	char *buf = new char[MAX_LINE_SIZE];
	char c;
    fpos_t fpos;
    fpos_t sofpos = 0;
	char *at;

	char *fnsrt = new char[MAX_PATH_LEN];
	char *fnpat = new char[MAX_PATH_LEN];
	char *fnidx = new char[MAX_PATH_LEN];

	strcpy(fnsrt,fullpath("db\\unsorted.txt"));
	strcpy(fnpat,fullpath("db\\patterns.txt"));
	strcpy(fnidx,fullpath("db\\index.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_LINE_SIZE,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"unsorted=",9)) strcpy(fnsrt,buf + 9);
			if (!strncmp(buf,"patterns=",9)) strcpy(fnpat,buf + 9);
			if (!strncmp(buf,"index=",6))    strcpy(fnidx,buf + 6);
		}
		fclose(fdirs);
	}

	FILE *funsort = fopen(fnsrt,"r");
	FILE *fsorted = fopen(fnpat,"w");
	FILE *findex = fopen(fnidx,"w");

	while (!feof(funsort)) {
		*buf = 0;
		at = fgets(buf,MAX_LINE_SIZE,funsort);
		if (*buf == '_') fprintf(fsorted,buf);
	}

	for (i = 'A';i <= ('Z' + 1);i++) {
	    fgetpos(fsorted,&fpos);
		fprintf(findex,"%li\n",fpos);
		fsetpos(funsort,&sofpos);
		while (!feof(funsort)) {
			*buf = 0;
			fgets(buf,MAX_LINE_SIZE,funsort);
			c = *buf;
			if (islower(c)) c += 'A' - 'a';
			if (i <= 'Z') {
				if (c == i) fprintf(fsorted,buf);
			} else if (c != '_') {
				if ((c < 'A') || (c > 'Z')) {
					if (*buf) fprintf(fsorted,buf);
				}
			}
		}
	}

	fclose(fsorted);
	fclose(findex);
	fclose(funsort);

	delete buf;
	delete fnsrt;
	delete fnpat;
	delete fnidx;
}

int CLearnDlg::cleanupaiml()
{
	int i;
	char *fnsrt = new char[MAX_PATH_LEN];
	char *fnpat = new char[MAX_PATH_LEN];
	char *fntmp = new char[MAX_PATH_LEN];
	char *buf = new char[MAX_LINE_SIZE];

	strcpy(fnsrt,fullpath("db\\unsorted.txt"));
	strcpy(fnpat,fullpath("db\\patterns.txt"));
	strcpy(fntmp,fullpath("db\\templates.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_BUF_LEN,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"unsorted=",9))   strcpy(fnsrt,buf + 9);
			if (!strncmp(buf,"patterns=",9))   strcpy(fnpat,buf + 9);
			if (!strncmp(buf,"templates=",10)) strcpy(fntmp,buf + 10);
		}
		fclose(fdirs);
	}

	FILE *fsorted = fopen(fnpat,"a");
	fprintf(fsorted,"<eof>\n");
	fclose(fsorted);

	FILE *ftemplates = fopen(fntmp,"a");
	fprintf(ftemplates,"\n");
	fclose(ftemplates);

	TCHAR szFileName[MAX_PATH_LEN];
	wsprintf(szFileName,TEXT("%hs"),fnsrt);
	i = DeleteFile(szFileName);

	delete buf;
	delete fnsrt;
	delete fnpat;
	delete fntmp;

	return !i;
}

void CLearnDlg::sortforstandardaiml()
{
	if (MessageBox(_T("This may take a very long time. Ok to continue?"),_T("Sorting to AIML standard"),MB_OKCANCEL) != IDOK) return;

	CStatusBar* status_bar = (CStatusBar *)theApp.GetMainWnd()->GetDlgItem(AFX_IDW_STATUS_BAR);
	status_bar->SetWindowText(_T("Sorting AIML patterns in database..."));

	TCHAR szFileName[MAX_PATH_LEN];
	TCHAR szOldFileName[MAX_PATH_LEN];
	char *buf = new char[MAX_LINE_SIZE];
	char *last = new char[MAX_LINE_SIZE];
	char *wrk[SORT_ITEMS];
	int w, x, iRet;
	bool notsorted = true;

	int cnt = 0;

	char *fnsrt = new char[MAX_PATH_LEN];
	char *fnpat = new char[MAX_PATH_LEN];

	strcpy(fnsrt,fullpath("db\\unsorted.txt"));
	strcpy(fnpat,fullpath("db\\patterns.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_LINE_SIZE,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"unsorted=",9)) strcpy(fnsrt,buf + 9);
			if (!strncmp(buf,"patterns=",9)) strcpy(fnpat,buf + 9);
		}
		fclose(fdirs);
	}

	for (w = 0;w < SORT_ITEMS;w++) wrk[w] = new char[MAX_LINE_SIZE];
	for (w = 0;w < SORT_ITEMS;w++) *wrk[w] = 0;

	wsprintf(szFileName,TEXT("%hs"),fnpat);
	wsprintf(szOldFileName,TEXT("%hs"),fnsrt);

	while (notsorted) {
		FILE *fpatn = fopen(fnpat,"r");
		FILE *fsort = fopen(fnsrt,"w");
		notsorted = false;
		strcpy(last,"_______________________________________________");
		w = 0;
		while (!feof(fpatn)) {
			fgets(buf,MAX_LINE_SIZE,fpatn);
			if (!*buf) continue;
			if (!strncmp(buf,"<eof>",5)) break;
			strcpy(wrk[w++],buf);
			if (w >= SORT_ITEMS) break;
		}
		while (!feof(fpatn)) {
			fgets(buf,MAX_LINE_SIZE,fpatn);
			if (!*buf) continue;
			if (!strncmp(buf,"<eof>",5)) break;
			for (x = 0,w = 1;w < SORT_ITEMS;w++)  if (strpcmp(wrk[w],wrk[x]) < 0) x = w;
			if (strpcmp(wrk[x],last) < 0) notsorted = true;
			strcpy(last,wrk[x]);
			fprintf(fsort,last);
			strcpy(wrk[x],buf);
		}
		for (x = 0;x < SORT_ITEMS;x++) {
			if (!*wrk[x]) continue;
			for (w = 0;w < SORT_ITEMS;w++) if (*wrk[w]) if (strpcmp(wrk[w],wrk[x]) < 0) x = w;
			if (strpcmp(wrk[x],last) < 0) notsorted = true;
			strcpy(last,wrk[x]);
			fprintf(fsort,last);
			*wrk[x] = 0;
			x = -1;
		}
		fprintf(fsort,"<eof>\n");
		fclose(fpatn);
		fclose(fsort);

		iRet = DeleteFile(szFileName);
		iRet = MoveFile(szOldFileName,szFileName);

		if (cnt++ > MAX_PASSES) { MessageBox(_T("Stopping early. Try again to sort some more!")); break; }
	}

	for (w = 0;w < SORT_ITEMS;w++) delete wrk[w];
	delete buf;
	delete last;
	delete fnsrt;
	delete fnpat;

	if (cnt++ > MAX_PASSES) status_bar->SetWindowText(_T("Partially Sorted AIML database...  Try again to sort some more!"));
	else status_bar->SetWindowText(_T("Completely Sorted AIML database for Standard Pattern Matching."));
}

void CLearnDlg::reindex()
{
	int i;
	char *buf = new char[MAX_LINE_SIZE];
	char c;
    fpos_t fpos;

	char *fnpat = new char[MAX_PATH_LEN];
	char *fnidx = new char[MAX_PATH_LEN];

	strcpy(fnpat,fullpath("db\\patterns.txt"));
	strcpy(fnidx,fullpath("db\\index.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_LINE_SIZE,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"patterns=",9)) strcpy(fnpat,buf + 9);
			if (!strncmp(buf,"index=",6))    strcpy(fnidx,buf + 6);
		}
		fclose(fdirs);
	}

	FILE *fpatrn = fopen(fnpat,"r");
	FILE *findex = fopen(fnidx,"w");

    fgetpos(fpatrn,&fpos);
	while (!feof(fpatrn)) {
		*buf = 0;
		fgets(buf,MAX_LINE_SIZE,fpatrn);
		if (*buf != '_') break;
	    fgetpos(fpatrn,&fpos);
	}
	c = *buf;
	if (islower(c)) c += 'A' - 'a';

	for (i = 'A';i <= 'Z';i++) {
		if (i <= c) fprintf(findex,"%li\n",fpos);
		if (i == c) {
			fsetpos(fpatrn,&fpos);
			while (!feof(fpatrn)) {
				*buf = 0;
				fgets(buf,MAX_LINE_SIZE,fpatrn);
				if (!strncmp(buf,"<eof>",5)) { c = 127; break; }
				c = *buf;
				if (islower(c)) c += 'A' - 'a';
				if (c != i) break;
				fgetpos(fpatrn,&fpos);
			}
		}
	}
	fprintf(findex,"%li\n",fpos);

	fclose(fpatrn);
	fclose(findex);

	delete buf;
	delete fnpat;
	delete fnidx;
}

void CLearnDlg::mergecategories()
{
	if (MessageBox(_T("This won't work correctly if AIML database has not been sorted. Ok to continue?"),_T("Sorting to AIML standard"),MB_OKCANCEL) != IDOK) return;

	CStatusBar* status_bar = (CStatusBar *)theApp.GetMainWnd()->GetDlgItem(AFX_IDW_STATUS_BAR);
	status_bar->SetWindowText(_T("Merging templates to remove duplicate AIML patterns in database..."));

	TCHAR szFileName[MAX_PATH_LEN];
	TCHAR szOldFileName[MAX_PATH_LEN];
	char *buf = new char[MAX_LINE_SIZE];
	char *last = new char[MAX_LINE_SIZE];
	char *dup = new char[MAX_LINE_SIZE];
	char *tmpl1 = new char[MAX_BUF_LEN];
	char *tmpl2 = new char[MAX_BUF_LEN];
	char *ntmp = new char[MAX_BUF_LEN];
	int iRet;
	char *at;
	char *atlast;
	char a;
	char b = '_';
	char c;
    fpos_t fpos;
    fpos_t tpos;
	bool blnDup = false;
	bool blnRnd = false;
	bool blnNew = true;

	int cnt = 0;

	char *fnsrt = new char[MAX_PATH_LEN];
	char *fnpat = new char[MAX_PATH_LEN];
	char *fnidx = new char[MAX_PATH_LEN];
	char *fntmp = new char[MAX_PATH_LEN];

	strcpy(fnsrt,fullpath("db\\unsorted.txt"));
	strcpy(fnpat,fullpath("db\\patterns.txt"));
	strcpy(fntmp,fullpath("db\\templates.txt"));
	strcpy(fnidx,fullpath("db\\index.txt"));

	FILE *fdirs = fopen(fullpath("options.ini"),"r");
	if (fdirs) {
		while (!feof(fdirs)) {
			fgets(buf,MAX_LINE_SIZE,fdirs);
			rtrim(buf);
			if (!strncmp(buf,"unsorted=",9))   strcpy(fnsrt,buf + 9);
			if (!strncmp(buf,"patterns=",9))   strcpy(fnpat,buf + 9);
			if (!strncmp(buf,"templates=",10)) strcpy(fntmp,buf + 10);
			if (!strncmp(buf,"index=",6))      strcpy(fnidx,buf + 6);
		}
		fclose(fdirs);
	}

	FILE *fpatn = fopen(fnpat,"r");
	FILE *fsort = fopen(fnsrt,"w");
	FILE *findx = fopen(fnidx,"w");
	FILE *ftmpl = fopen(fntmp,"a+");

	*last = 0;
	while (!feof(fpatn)) {
		*buf = 0;
		fgets(buf,MAX_LINE_SIZE,fpatn);
		if (!*buf) continue;
		if (!strncmp(buf,"<eof>",5)) break;
		if (!(at = strchr(buf,'@'))) { fprintf(fsort,buf); continue; }
		if (*last) {
			atlast = strchr(last,'@');
			if (atlast) a = '@';
			else {
				atlast = strchr(last,'!');
				a = '!';
			}
			at = strchr(buf,'@');
			if (at) {
				*atlast = 0;
				*at = 0;
				if (!strcmp(last,buf) && (a == '@' || blnDup)) {
					// duplicate patterns
					strcpy(dup,buf);
					if (blnDup) {
						strcpy(tmpl2,at + 1);
						rtrim(tmpl2);
						tpos = atol(tmpl2);
						fsetpos(ftmpl,&tpos);
						fgets(tmpl2,MAX_BUF_LEN,ftmpl);
						rtrim(tmpl2);
						expand(tmpl2);
						if (blnRnd || strcmp(tmpl1,tmpl2)) {
							blnRnd = true;
							strcat(ntmp,"<li>");
							strcat(ntmp,tmpl2);
							strcat(ntmp,"</li>");
						}
					} else {
						cnt++;
						if (blnNew) {
							getdts(tmpl1);
							fprintf(ftmpl,"==*** Merged Categories (%s) ***==\n",tmpl1);
							blnNew = false;
						}

						strcpy(tmpl1,atlast + 1);
						rtrim(tmpl1);
						tpos = atol(tmpl1);
						fsetpos(ftmpl,&tpos);
						fgets(tmpl1,MAX_BUF_LEN,ftmpl);
						rtrim(tmpl1);
						expand(tmpl1);

						strcpy(tmpl2,at + 1);
						rtrim(tmpl2);
						tpos = atol(tmpl2);
						fsetpos(ftmpl,&tpos);
						fgets(tmpl2,MAX_BUF_LEN,ftmpl);
						rtrim(tmpl2);
						expand(tmpl2);

						if (strcmp(tmpl1,tmpl2)) {
							blnRnd = true;
							sprintf(ntmp,"<random><li>%s</li><li>%s</li>",tmpl1,tmpl2);
						} else {
							blnRnd = false;
							strcpy(ntmp,tmpl1);
						}
					}
					*atlast = '!';
					*at = '!';
					blnDup = true;
				} else {
					*atlast = a;
					*at = '@';
					if (blnDup) {
						fseek(ftmpl,0,SEEK_END);
						fgetpos(ftmpl,&tpos);
						fprintf(fsort,dup);
						fprintf(fsort,"@%li\n",tpos);
						if (blnRnd) strcat(ntmp,"</random>");
						compress(ntmp);
						fprintf(ftmpl,"%s\n",ntmp);
					}
					blnDup = false;
				}
			}
			c = *last;
			if (islower(c)) c += 'A' - 'a';
			if (b != c && b != '*') {
				fgetpos(fsort,&fpos);
				fprintf(findx,"%li\n",fpos);
				if (b == '_') b = 'A'; else b++;
				if (c < 'A' || c > 'Z') c = 'Z' + 1;
				while (b < c) {
					fprintf(findx,"%li\n",fpos);
					b++; 
				}
				if (b > 'Z') c = b = '*';
			}
			fprintf(fsort,last);
			strcpy(last,buf);
		} else strcpy(last,buf);
	}

	if (*last) {
		c = *last;
		if (islower(c)) c += 'A' - 'a';
		if (b != c && b != '*') {
			fgetpos(fsort,&fpos);
			fprintf(findx,"%li\n",fpos);
			if (b == '_') b = 'A'; else b++;
			if (c < 'A' || c > 'Z') c = 'Z' + 1;
			while (b < c) {
				fprintf(findx,"%li\n",fpos);
				b++; 
			}
		}
		fprintf(fsort,last);
	}
	if (blnDup) {
		fseek(ftmpl,0,SEEK_END);
		fgetpos(ftmpl,&tpos);
		fprintf(fsort,dup);
		fprintf(fsort,"@%li\n",tpos);
		if (blnRnd) strcat(ntmp,"</random>");
		compress(ntmp);
		fprintf(ftmpl,"%s\n",ntmp);
	}

	fprintf(fsort,"<eof>\n");
	fclose(fpatn);
	fclose(fsort);
	fclose(findx);
	fclose(ftmpl);

	wsprintf(szFileName,TEXT("%hs"),fnpat);
	wsprintf(szOldFileName,TEXT("%hs"),fnsrt);

	iRet = DeleteFile(szFileName);
	iRet = MoveFile(szOldFileName,szFileName);

	delete buf;
	delete last;
	delete tmpl1;
	delete tmpl2;
	delete ntmp;
	delete fnsrt;
	delete fnpat;
	delete fntmp;
	delete fnidx;

	status_bar->SetWindowText(_T("Completely removed duplicate patterns and merged the corresponding templates in the AIML database."));
}
