/* Copyright (c) 2000-2002, Jacco Bikker
 * Copyright (c) 2002-2003, Gary Dubuque
 *
 * This file is part of NP+AIML.
 * NP+AIML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * NP+AIML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with NP+AIML; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

// SubjectsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AIMLpad.h"
#include "SubjectsDlg.h"
#include "CasesDlg.h"
#include "pu.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSubjectsDlg dialog


CSubjectsDlg::CSubjectsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSubjectsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSubjectsDlg)
	m_subject = _T("");
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CSubjectsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSubjectsDlg)
	DDX_Control(pDX, IDC_SUBJECTLIST, m_subjects);
	DDX_Text(pDX, IDC_SUBJECTNAME, m_subject);
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

void CSubjectsDlg::FillList()
{
	CString key;
	Subject *pSubject;

	m_subjects.ResetContent();

	POSITION pos = kb->m_mapStringToSubject.GetStartPosition();
	while (pos != NULL) {
		kb->m_mapStringToSubject.GetNextAssoc(pos,key,pSubject);
		m_subjects.AddString(pSubject->m_name);
	}

	UpdateData(false);
}

BOOL CSubjectsDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	FillList();

	CWnd* pWnd = GetDlgItem(IDC_SUBJECTNAME);
	pWnd->SetFocus();

	return false;
}

BEGIN_MESSAGE_MAP(CSubjectsDlg, CDialog)
	//{{AFX_MSG_MAP(CSubjectsDlg)
	ON_BN_CLICKED(IDC_CONSIDER, OnConsider)
	ON_BN_CLICKED(IDC_DISCARD, OnDiscard)
	ON_LBN_DBLCLK(IDC_SUBJECTLIST, OnDblclkSubjectlist)
	ON_BN_CLICKED(IDC_GET_SUBJECT, OnGetSubject)
	ON_BN_CLICKED(IDC_PUT_SUBJECT, OnPutSubject)
	ON_BN_CLICKED(IDC_LINK_CASES, OnLinkCases)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSubjectsDlg message handlers

void CSubjectsDlg::OnGetSubject() 
{
	char *fn = new char[MAX_PATH_LEN];
	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(true, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Load Subject From File...");

	if (dlg.DoModal() != IDCANCEL)	{
		GetParent()->BeginWaitCursor();
		sprintf(fn,"\"%s\"",m_subject);
		pu->utconsdr(fn);
		sprintf(fn,"\"%s\"",dlg.GetPathName());
		pu->utget(fn);
		GetParent()->EndWaitCursor();
	}
	delete dlg;
	delete fn;

	FillList();
}

void CSubjectsDlg::OnPutSubject() 
{
	char *fn = new char[MAX_PATH_LEN];
	CString szFilter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";
	CFileDialog dlg(false, _T("txt"), NULL, OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY, szFilter, NULL);

	dlg.m_ofn.lpstrTitle = TEXT("Save Subject To File...");

	if (dlg.DoModal() != IDCANCEL)	{
		GetParent()->BeginWaitCursor();
		sprintf(fn,"\"%s\"",m_subject);
		pu->utconsdr(fn);
		sprintf(fn,"\"%s\"",dlg.GetPathName());
		pu->utput(fn);
		GetParent()->EndWaitCursor();
	}

	delete dlg;
	delete fn;
}

void CSubjectsDlg::OnConsider() 
{
	char *buf;

	UpdateData(true);

	buf = new char[m_subject.GetLength() + 2];
	sprintf(buf,"%s",m_subject);
	if (!kb->Find(buf)) {
		if (MessageBox(_T("Ok to add new subject area?"),m_subject,MB_YESNO) == IDYES) pu->utconsdr(buf);
	}
	delete buf;

	FillList();
}

void CSubjectsDlg::OnDiscard() 
{
	char *buf;

	UpdateData(true);

	buf = new char[m_subject.GetLength() + 2];
	sprintf(buf,"%s",m_subject);
	if (kb->Find(buf)) {
		if (MessageBox(_T("Ok to delete subject area and all cases and facts related to it?"),m_subject,MB_YESNO) == IDYES) {
			m_subject = _T("");
			if (strcmp(buf,"New Cases")) {
				if (kb->Remove(buf)) if (kb->Find("New Cases")) m_subject = _T("New Cases");
			} else {
				kb->curSub->m_cases.RemoveAll(); 
				kb->curSub->m_cases.AddGbl();
			}
		}
	} else m_subject = _T("");
	delete buf;

	FillList();
}

void CSubjectsDlg::OnDblclkSubjectlist() 
{
	int nIndex = m_subjects.GetCurSel();
	if (nIndex != LB_ERR) {
		m_subjects.GetText(nIndex,m_subject);
		UpdateData(false);
	}
}

void CSubjectsDlg::OnLinkCases() 
{
	Subject *pSubject = pu->kb.curSub;
	char *buf;
	CCasesDlg* m_pCaseDialog = new CCasesDlg;

	UpdateData(true);

	buf = new char[m_subject.GetLength() + 2];
	sprintf(buf,"%s",m_subject);
	if (kb->Find(buf)) {
		m_pCaseDialog->pu = pu;
		m_pCaseDialog->kb = &(pu->kb);

		m_pCaseDialog->m_subject = pu->kb.curSub->m_name;
		m_pCaseDialog->m_case = pu->kb.curSub->m_cases.curCase->m_name;
		
		m_pCaseDialog->DoModal();
	}
	pu->kb.curSub = pSubject;

	delete m_pCaseDialog;
	delete buf;
}
