/* Copyright (c) 2000-2002, Jacco Bikker
 * Copyright (c) 2002-2003, Gary Dubuque
 * Copyright (c) 2003-2004, Kino Coursey (Mods for CyN)
 *
 * This file is part of AIMLpad.
 * AIMLpad is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * AIMLpad is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with AIMLpad; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */
// core.h: interface for the core class.
//
//////////////////////////////////////////////////////////////////////
#ifndef __INC_CORE_H__
#define __INC_CORE_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MAX_STARS 25
#define MAX_CACHE 33

#include "sys/types.h"
#include "time.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"

#include "pu.h"
#include "nodes.h"

#include "support.h"

class PUvm;

class idx;

class Core  
{
public:
	Core();
	virtual ~Core();
	
// Operations
public:
	void loadchunks();
	void initialize();
	char *getsize();
	char *getbot(char *varname);
	char *getvar(char *varname,char *defval);
	void setvar(char *varname,char *value);
	void setname(char *name);
	void execute(char *command, char *reply);
	void updatemood(char *text);
	int calcmood(char *text);
	void logerror(char *tag,char *text);
	void savetarget(char *ti_input,char *ti_that,char *ti_topic,char *tm_pattern,char *tm_that,char *tm_topic,char *tm_template,char *reply);
	void savegossip(char *text);
	void substitute(char *text,char *fn,bool both);
	void tidyup(char *text);
	void tidyup_ps(char *text);
	bool match(char *text,char *pattern,int final);
	bool testguard(fpos_t templpos);
	fpos_t interpret(char *text);
	fpos_t gm_interpret(char *text);
	void loadgm();
	void unloadgm();
	void reloadgm();
	void respond(char *text, char* reply);
	void reevaluate(char *line, char *reply);
	char *docmds(char *text,char *ans);
	char *doatomic(char *tag);
	char *donode(char *tag,char *text);
	char *donested(char *text);
	char *doSRAI(char *text);
	char *doSet(char *tag,char *text);
	char *doCondition(char *tag,char *text);
	char *doRandom(char *text);
	char *doUppercase(char *text);
	char *doLowercase(char *text);
	char *doFormalcase(char *text);
	char *doSentencecase(char *text);
	char *doThink(char *text);
	char *doPerson(char *text,char *fname);
	char *doSystem(char *text);
	char *doScript(char *text);
	char *doGossip(char *text);
	char *doLearn(char *text);
	char *doUseSaved(char *text);
	char *doLi(char *tag,char *text);
	char *doDefault(char *text);

	char *doCycCondition(char *tag,char *text);
	char *doCycSystem(char *text);
	char *doCycTerm(char *text);
	char *doCycRandom(char *text);
	char *doCycRetract(char *text);
	char *doCycAssert(char *text);
	bool lookupCycTerm(char *cyctemplate, char *text);

	bool getAnswer(char *request, char *reply);
	void runIrc();

	bool newsec;
	char *second;
	bool clrscrn;
	bool logtarget;
	bool usegraphmaster;
	bool useconnect;
	bool stopbot;
	bool nosave;
	bool autoname;
	bool dbfnd;
	bool usetermcache;

	int intAgent;
	bool autocyc;
	bool ircactive;

	char fnini[MAX_PATH_LEN];
	char fnchr[MAX_PATH_LEN];
	char fnmds[MAX_PATH_LEN];
	char fndat[MAX_PATH_LEN];
	char fnbot[MAX_PATH_LEN];
	char fnwrk[MAX_PATH_LEN];
	char fnidx[MAX_PATH_LEN];
	char fnsrt[MAX_PATH_LEN];
	char fnpat[MAX_PATH_LEN];
	char fntmp[MAX_PATH_LEN];
	char fninf[MAX_PATH_LEN];
	char fnerr[MAX_PATH_LEN];
	char fntar[MAX_PATH_LEN];
	char fngos[MAX_PATH_LEN];
	char fnsub[MAX_PATH_LEN];
	char fnper[MAX_PATH_LEN];
	char fnper2[MAX_PATH_LEN];
	char fnperf[MAX_PATH_LEN];
	char fngndr[MAX_PATH_LEN];
	char fnaiml[MAX_PATH_LEN];
	char fnmore[MAX_PATH_LEN];
	char fnquik[MAX_PATH_LEN];
	char fnagnt[MAX_PATH_LEN];
	char fnagntTTS[MAX_PATH_LEN];
    char urlcycget[MAX_PATH_LEN];
	char urlcycput[MAX_PATH_LEN];

	PUvm *pu;

	char *topic;
	char *client;

	char *inputs[5];
	char *thats[5];
	char *stars[MAX_STARS];
	char *thatstars[MAX_STARS];
	char *topicstars[MAX_STARS];
	
	char *termcache_key[MAX_CACHE];
	char *termcache_value[MAX_CACHE];
	time_t termcache_timestamp[MAX_CACHE];

	int stars_stkidx;
	int thatstars_stkidx;
	int topicstars_stkidx;

//	int  stars_at[MAX_STARS];
//	char stars_type[MAX_STARS]; 

	fpos_t chunk[27];

private:
	int recursion;
	bool exitplease;

	float happyness;
	float helpfullness;
	float humor;
	float affection;
	float trust;
	float interest;

	char *saves[10];
	int sage[10];

	char *best;
	char *bestthat;
	char *besttopic;

	char *wrk;
	char *varvalue;

	idx *gm;
};

#endif // __INC_CORE_H__
