/* Copyright (c) 2000-2002, Jacco Bikker
 * Copyright (c) 2002-2003, Gary Dubuque
 *
 * This file is part of NP+AIML.
 * NP+AIML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * NP+AIML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with NP+AIML; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

// facts.h: interface for the Facts class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FACTS_H__A114551C_A0AA_4FAE_8710_CA74363F6EEC__INCLUDED_)
#define AFX_FACTS_H__A114551C_A0AA_4FAE_8710_CA74363F6EEC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Afxtempl.h"

#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "math.h"

#include "support.h"

//
// FACTS
//

class Fact : public CObject
{
public:
	Fact();
	Fact(char *attr,char *value,char *source,double cf);

// Attributes
public:
	CString m_attribute;
	CString m_value;
	CString m_source;
	double m_cf;
	CString m_timestamp;
	CStringList m_mvlist;

// Operations
public:
	void Push(char *value);
	void Queue(char *value);
	bool Pop(char *value);
	bool Find(int at, char *value);
	bool Insert(int at,char *value);
	bool Extract(int at, char *value);
	void FormatFact(CString& str);
	void XMLFact(CString& str);

// Implementation
public:
	virtual ~Fact();
};

typedef CTypedPtrMap<CMapStringToOb,CString,Fact*> CMapStringToFact;

class Facts  
{
public:
	Facts();

// Operations
public:
	void Add(char *attr,char *value,char *source,double cf);
	void Add(char *attr,char *value);
	void Put(CString attr,CString value,CString source,double cf);
	void RemoveAll();
	bool Remove(char *attr);
	bool Seek(CString attr);
	bool Find(char *attr);
	bool GetFirst();
	bool GetNext();
	void FormatFacts(CString& str);
	void XMLFacts(CString& str);

// Attributes
public:
	CMapStringToFact m_mapStringToFact;

	POSITION At;
	CString Key;
	Fact *curFact;

// Implementation
public:
	virtual ~Facts();
};

//
// CASES
//

class Case : public CObject
{
public:
	Case();
	Case(char *name);

// Attributes
public:
	CString m_name;
	Facts m_facts;
	CString m_timestamp;

// Operations
public:
	void FormatCase(CString& str);
	void XMLCase(CString& str);

// Implementation
public:
	virtual ~Case();
};

typedef CTypedPtrMap<CMapStringToOb,CString,Case*> CMapStringToCase;

class Cases  
{
public:
	Cases();

// Operations
public:
	void Add(char *name);
	void AddGbl();
	void AddShare();
	void RemoveAll();
	bool Remove(char *name);
	bool Find(char *name);
	bool Seek(CString name);
	bool GetFirst();
	bool GetNext();
	void ListCases(CString& str);
	void FormatCases(CString& str);
	void FileCases(CString& str);
	void XMLCases(CString& str);

// Attributes
public:
	CMapStringToCase m_mapStringToCase;

	POSITION At;
	CString Key;
	Case *curCase;
	Case *curGbl;

// Implementation
public:
	virtual ~Cases();
};

//
// SUBJECTS (topics)
//

class Subject : public CObject
{
public:
	Subject(char *name);

// Attributes
public:
	CString m_name;
	Cases m_cases;
	CString m_timestamp;

// Operations
public:
	void FormatSubject(CString& str);
	void XMLSubject(CString& str);

// Implementation
public:
	virtual ~Subject();
};

typedef CTypedPtrMap<CMapStringToOb,CString,Subject*> CMapStringToSubject;

class Subjects  
{
public:
	Subjects();

// Operations
public:
	void Add(char *name);
	void AddLoc(char *name);
	void AddAct(char *name);
	void AddTag(char *name);
	void RemoveAll();
	bool Remove(char *name);
	bool Find(char *name);
	bool GetFirst();
	bool GetNext();
	void ListSubjects(CString& str);
	void FormatSubjects(CString& str);
	void XMLSubjects(CString& str);

	bool getValue(char *name,char *value);
	bool putValue(char *name,char *value);
	bool putGlobal(char *name,char *value);
	bool putShare(char *name,char *value);
	bool delValue(char *name);
	Fact *getFact(CString name);

	char *calc(char *exp);

// Attributes
public:
	CMapStringToSubject m_mapStringToSubject;

	POSITION At;
	CString Key;
	Subject *curSub;
	Subject *curLoc;
	Subject *curTag;

private:
	char *varname(char *a,char **t);
	int stkint(char *p);
	double stknum(char *p);
	char *stksav(double z);
	int fourbang(int h,int op);
	int funcbang(int h,int op);
	int confunc(int h);
	int lenfunc(int h);
	int trimfunc(int h);
	int numfunc(int h);
	int decfunc(int h);
	int padfunc(int h);
	int justfunc(int h);
	int zerofunc(int h);
	int midfunc(int h);
	int fldfunc(int h);
	int chgfunc(int h);
	int cntfunc(int h);
	int replacefunc(int h);
	int tagfunc(int h);
	int attrfunc(int h);
	int datafunc(int h);
	int datefunc(int h);
	int findfunc(int h);
	int leftfunc(int h);
	int rightfunc(int h);
	int subscript(int h);
	int getfunc(char *fn);
	int rollup(char *s);
	void rollout(int tos,char **s);

	char dlm[32];
	char *hold[64];
	char *op1[64];
	int op2[64];
	int push[64];

// Implementation
public:
	virtual ~Subjects();

};

#endif // !defined(AFX_FACTS_H__A114551C_A0AA_4FAE_8710_CA74363F6EEC__INCLUDED_)
