/* Copyright (c) 2003, Gary Dubuque
 * Copyright (c) 2003-2004, Kino Coursey (Mods for CyN)
 *
 * This file is part of AIMLpad.
 * AIMLRight is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2, or (at your option) any later 
 * version.
 *
 * AIMLpad is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.

 * You should have received a copy of the GNU General Public License along 
 * with AIMLpad; see the file COPYING. If not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

// nodes.h: interface for the Graphmaster classes.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NODES_H_INCLUDED_)
#define AFX_NODES_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Afxtempl.h"
#include "core.h"
#include "support.h"

class Core;

// Templatelinks

class TemplateLink
{
public:
	TemplateLink();
	virtual ~TemplateLink();
public:
	void Add(fpos_t new_template);
	void RemoveAll();
	//Attributes
public:
 fpos_t templatepos;
 TemplateLink *Next;
};



//
// NODES
//

class Node  
{
public:
	Node();
	virtual ~Node();

// Operations
public:
	void Add(CString key);
	void RemoveAll();
	bool Remove(CString key);
	bool Find(CString key);
	bool GetFirst();
	bool GetNext();

// Attributes
public:
	POSITION At;
	CString Key;
	Node *curWord;
	CTypedPtrMap<CMapStringToOb,CString,Node*> m_WordNodes;

	fpos_t m_template;
	TemplateLink *TemplateChain;
};

//
// The Match Results class
//

class Match
{
public:
	Match();
	~Match();

	void addInputStar(CString s);
	void addThatStar(CString s);
	void addTopicStar(CString s);
	CString getInputStar(int i);
	CString getThatStar(int i);
	CString getTopicStar(int i);
	int getInputStarCount();
	int getThatStarCount();
	int getTopicStarCount();

	Node *m_node;

	CString inputPattern;
	CString thatPattern;
	CString topicPattern;
	CString inputPath;
	CString thatPath;
	CString topicPath;

private:
	CStringList inputStar;
	CStringList thatStar;
	CStringList topicStar;
};

//
// The whole Graph
//

class idx
{
public:
	idx();
	virtual ~idx();

public:
	Core *bot;
	Node *root;
	long m_size;

	bool Add(char *path,fpos_t tmplt);
	Match *Select(CString path,Core *World);
	bool Load(char *fname);

private:
	Node *add(Node *parent,CString phrase);
	Match *select(Node *parent,Node *child,int part,CString input,CString star,CString path);
	Match *select2(Node *parent,Node *child,int part,CString input,CString star,CString path,Core *World);
	void addStar(Match *m,CString value,int part);
	void addPath(Match *m,CString value,int part);
	bool testguard(Match *m,Core *World);
	int pushStar(Core *World,CString value,int part);
	void popStar(Core *World,int value,int part);
};

#endif // !defined(AFX_NODES_H_INCLUDED_)
