/*
   
   wn.h - header file needed to use WordNet Run Time Library

*/

#ifndef _WN_
#define _WN_

// C RunTime Header Files

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

#include "support.h"

#ifndef NULL
#define NULL	0
#endif

#define DICTDIR         "\\Dict"
#define DATAFILE		"%s\\%s.dat"
#define INDEXFILE		"%s\\%s.idx"
#define SENSEIDXFILE	"%s\\sense.idx"
#define KEYIDXFILE		"%s\\key.idx"
#define REVKEYIDXFILE	"%s\\revkey.idx"
#define VRBSENTFILE  	"%s\\sents.vrb"
#define VRBIDXFILE		"%s\\sentidx.vrb"
#define CNTLISTFILE     "%s\\cntlist.rev"
#define EXCFILE			"%s\\%s.exc"

/* Various buffer sizes */

#define KEY_LEN		(1024)
#define LINE_LEN	(1024*25)


#define SEARCHBUF	((long)(200*(long)1024))
#define LINEBUF		(15*1024) /* 15K buffer to read index & data files */
#define SMLINEBUF	(3*1024) /* small buffer for output lines */
#define WORDBUF		(256)	/* buffer for one word or collocation */

#define ALLSENSES	0	/* pass to findtheinfo() if want all senses */
#define MAXID		15	/* maximum id number in lexicographer file */
#define MAXDEPTH	20	/* maximum tree depth - used to find cycles */
#define MAXSENSE	75	/* maximum number of senses in database */
#define MAX_FORMS	5	/* max # of different 'forms' word can have */
#define MAXFNUM		44	/* maximum number of lexicographer files */

/* Pointer type and search type counts */

/* Pointers */

#define ANTPTR           1	/* ! */
#define HYPERPTR         2	/* @ */
#define HYPOPTR          3	/* ~ */
#define ENTAILPTR        4	/* * */
#define SIMPTR           5	/* & */

#define ISMEMBERPTR      6	/* #m */
#define ISSTUFFPTR       7	/* #s */
#define ISPARTPTR        8	/* #p */

#define HASMEMBERPTR     9	/* %m */
#define HASSTUFFPTR     10	/* %s */
#define HASPARTPTR      11	/* %p */

#define MERONYM         12	/* % (not valid in lexicographer file) */
#define HOLONYM         13	/* # (not valid in lexicographer file) */
#define CAUSETO         14	/* > */
#define PPLPTR	        15	/* < */
#define SEEALSOPTR	16	/* ^ */
#define PERTPTR		17	/* \ */
#define ATTRIBUTE	18	/* = */
#define VERBGROUP	19	/* $ */
#define NOMINALIZATIONS 20	/* + */
#define CLASSIFICATION  21	/* ; */
#define CLASS           22	/* - */

#define LASTTYPE	CLASS

/* Misc searches */

#define SYNS            (LASTTYPE + 1)
#define FREQ            (LASTTYPE + 2)
#define FRAMES          (LASTTYPE + 3)
#define COORDS          (LASTTYPE + 4)
#define RELATIVES	(LASTTYPE + 5)
#define HMERONYM        (LASTTYPE + 6)
#define HHOLONYM	(LASTTYPE + 7)
#define WNGREP		(LASTTYPE + 8)
#define OVERVIEW	(LASTTYPE + 9)

#define MAXSEARCH       OVERVIEW

/* Specific nominalization pointers */

#define NOMIN_START     (OVERVIEW + 1)

#define NOMIN_V_ATE     (NOMIN_START)          /* +a */
#define NOMIN_V_IFY     (NOMIN_START + 1)      /* +b */
#define NOMIN_V_ISE_IZE (NOMIN_START + 2)      /* +c */
#define NOMIN_ACY       (NOMIN_START + 3)      /* +d */
#define NOMIN_AGE       (NOMIN_START + 4)      /* +e */
#define NOMIN_AL        (NOMIN_START + 5)      /* +f */
#define NOMIN_ANCE_ENCE (NOMIN_START + 6)      /* +g */
#define NOMIN_ANCY_ENCY (NOMIN_START + 7)      /* +h */
#define NOMIN_ANT_ENT   (NOMIN_START + 8)      /* +i */
#define NOMIN_ARD       (NOMIN_START + 9)      /* +j */
#define NOMIN_ARY       (NOMIN_START + 10)     /* +k */
#define NOMIN_ATE       (NOMIN_START + 11)     /* +l */
#define NOMIN_ATION     (NOMIN_START + 12)     /* +m */
#define NOMIN_EE        (NOMIN_START + 13)     /* +n */
#define NOMIN_ER        (NOMIN_START + 14)     /* +o */
#define NOMIN_ERY_RY    (NOMIN_START + 15)     /* +p */
#define NOMIN_ING_INGS  (NOMIN_START + 16)     /* +q */
#define NOMIN_ION       (NOMIN_START + 17)     /* +r */
#define NOMIN_IST       (NOMIN_START + 18)     /* +s */
#define NOMIN_MENT      (NOMIN_START + 19)     /* +t */
#define NOMIN_OR        (NOMIN_START + 20)     /* +u */
#define NOMIN_URE       (NOMIN_START + 21)     /* +v */
#define NOMIN_MISC      (NOMIN_START + 22)     /* +w */
#define NOMIN_UNMARKED  (NOMIN_START + 23)     /* +x */

#define NOMIN_END       NOMIN_UNMARKED

#define CLASSIF_START    (NOMIN_END + 1)

#define CLASSIF_CATEGORY (CLASSIF_START)        /* ;c */
#define CLASSIF_USAGE    (CLASSIF_START + 1)    /* ;u */
#define CLASSIF_REGIONAL (CLASSIF_START + 2)    /* ;r */

#define CLASSIF_END      CLASSIF_REGIONAL

#define CLASS_START      (CLASSIF_END + 1)

#define CLASS_CATEGORY   (CLASS_START)          /* -c */
#define CLASS_USAGE      (CLASS_START + 1)      /* -u */
#define CLASS_REGIONAL   (CLASS_START + 2)      /* -r */

#define CLASS_END        CLASS_REGIONAL

#define MAXPTR          CLASS_END

/* WordNet part of speech stuff */

#define NUMPARTS	4	/* number of parts of speech */
#define NUMFRAMES	35	/* number of verb frames */

#define NUMPREPS	15

/* Generic names for part of speech */

#define NOUN		1
#define VERB		2
#define ADJ			3
#define ADV			4
#define SATELLITE	5	/* not really a part of speech */
#define ADJSAT		SATELLITE

#define ALL_POS		0	/* passed to in_wn() to check all POS */

#define bit(n) ((unsigned int)((unsigned int)1<<((unsigned int)n)))

/* Adjective markers */

#define PADJ		1	/* (p) */
#define NPADJ		2	/* (a) */
#define IPADJ		3	/* (ip) */

#define UNKNOWN_MARKER		0
#define ATTRIBUTIVE			NPADJ
#define PREDICATIVE			PADJ
#define IMMED_POSTNOMINAL	IPADJ

/* For adjectives, indicates synset type */

#define DONT_KNOW	0
#define DIRECT_ANT	1	/* direct antonyms (cluster head) */
#define INDIRECT_ANT	2	/* indrect antonyms (similar) */
#define PERTAINYM	3	/* no antonyms or similars (pertainyms) */

/* Flags for printsynset() */

#define ALLWORDS	0	/* print all words */
#define SKIP_ANTS	0	/* skip printing antonyms in printsynset() */
#define PRINT_ANTS	1	/* print antonyms in printsynset() */
#define SKIP_MARKER	0	/* skip printing adjective marker */
#define PRINT_MARKER	1	/* print adjective marker */

/* Trace types used by printspaces() to determine print sytle */

#define TRACEP		1	/* traceptrs */
#define TRACEC		2	/* tracecoords() */
#define TRACEI		3	/* traceinherit() */

#define DEFON 1
#define DEFOFF 0

#define TMPBUFSIZE 1024*10

#define MAX_TRIES	5

/* Simple hash function */
#define HASHTABSIZE	1223	/* Prime number. Must be > 2*MAXTOPS */
#define hash(n) ((n) % HASHTABSIZE)


/* Data structures used by search code functions. */

/* Structure for index file entry */
typedef struct {
    long idxoffset;		/* byte offset of entry in index file */
    char *wd;			/* word string */
    char *pos;			/* part of speech */
    int sense_cnt;		/* sense (collins) count */
    int off_cnt;		/* number of offsets */
    int tagged_cnt;		/* number senses that are tagged */
    unsigned long *offset;	/* offsets of synsets containing word */
    int ptruse_cnt;		/* number of pointers used */
    int *ptruse;		/* pointers used */
} Index, *IndexPtr;

/* Structure for data file synset */
typedef struct ss {
    long hereiam;		/* current file position */
    int sstype;			/* type of ADJ synset */
    int fnum;			/* file number that synset comes from */
    char *pos;			/* part of speech */
    int wcount;			/* number of words in synset */
    char **words;		/* words in synset */
    int *lexid;			/* unique id in lexicographer file */
    int *wnsns;			/* sense number in wordnet */
    int whichword;		/* which word in synset we're looking for */
    int ptrcount;		/* number of pointers */
    int *ptrtyp;		/* pointer types */
    long *ptroff;		/* pointer offsets */
    int *ppos;			/* pointer part of speech */
    int *pto;			/* pointer 'to' fields */
    int *pfrm;			/* pointer 'from' fields */
    int fcount;			/* number of verb frames */
    int *frmid;			/* frame numbers */
    int *frmto;			/* frame 'to' fields */
    char *defn;			/* synset gloss (definition) */
    unsigned int key;	/* unique synset key */

    /* these fields are used if a data structure is returned
       instead of a text buffer */

    struct ss *nextss;		/* ptr to next synset containing searchword */
    struct ss *nextform;	/* ptr to list of synsets for alternate spelling of wordform */
    int searchtype;			/* type of search performed */
    struct ss *ptrlist;		/* ptr to synset list result of search */
    char *headword;			/* if pos is "s", this is cluster head word */
    short headsense;		/* sense number of headword */
} Synset, *SynsetPtr;

typedef struct si {
    char *sensekey;		/* sense key */
    char *word;			/* word string */
    long loc;			/* synset offset */
    int wnsense;		/* WordNet sense number */
    int tag_cnt;		/* number of semantic tags to sense */
    struct si *nextsi;	/* ptr to next sense index entry */
} SnsIndex, *SnsIndexPtr;
    
typedef struct {
    int SenseCount[MAX_FORMS];	/* number of senses word form has */
    int OutSenseCount[MAX_FORMS]; /* number of senses printed for word form */
    int numforms;		/* number of word forms searchword has */
    int printcnt;		/* number of senses printed by search */
    char *searchbuf;		/* buffer containing formatted results */
    SynsetPtr searchds;		/* data structure containing search results */
} SearchResults, *SearchResultsPtr;

/* Primry search algorithm for use with user interfaces */
char *findtheinfo(char *, int, int, int);	

/* Primary search algorithm for use with programs (returns data structure) */
SynsetPtr findtheinfo_ds(char *, int, int, int); 

/* Set bit for each search type that is valid for the search word
   passed and return bit mask. */
unsigned int is_defined(char *, int); 

/* Set bit for each POS that search word is in.  0 returned if
   word is not in WordNet. */
unsigned int in_wn(char *, int);	

/* Find word in index file and return parsed entry in data structure.
   Input word must be exact match of string in database. */
IndexPtr index_lookup(char *, int); 

/* 'smart' search of index file.  Find word in index file, trying different
   techniques - replace hyphens with underscores, replace underscores with
   hyphens, strip hyphens and underscores, strip periods. */
IndexPtr getindex(char *, int);	
IndexPtr parse_index(long, int, char *);

/* Read synset from data file at byte offset passed and return parsed
   entry in data structure. */
SynsetPtr read_synset(int, long, char *);

/* Read synset at current byte offset in file and return parsed entry
   in data structure. */
SynsetPtr parse_synset(FILE *, int, char *); 

/* Free a synset linked list allocated by findtheinfo_ds() */
void free_syns(SynsetPtr);	

/* Free a synset */
void free_synset(SynsetPtr);	

/* Free an index structure */
void free_index(IndexPtr);	

/* Recursive search algorithm to trace a pointer tree and return results
   in linked list of data structures. */
SynsetPtr traceptrs_ds(SynsetPtr, int, int, int);

/* Do requested search on synset passed, returning output in buffer. */
char *do_trace(SynsetPtr, int, int, int);

/*** Morphology functions (morph.c) ***/

/* Open exception list files */
int morphinit();	

/* Close exception list files and reopen */
int re_morphinit();	

/* Try to find baseform (lemma) of word or collocation in POS. */
char *morphstr(char *, int);	

/* Try to find baseform (lemma) of individual word in POS. */
char *morphword(char *, int);	

/*** Utility functions (wnutil.c) ***/

/* Top level function to open database files, initialize wn_filenames,
   and open exeception lists. */
int wninit();		

/* Top level function to close and reopen database files, initialize
   wn_filenames and open exception lists. */
int re_wninit();	

/* Count the number of underscore or space separated words in a string. */
int cntwords(char *, char);		

/* Convert string to lower case remove trailing adjective marker if found */
char *strtolower(char *);	

/* Convert string passed to lower case */
char *ToLowerCase(char *);	

/* Replace all occurrences of 'from' with 'to' in 'str' */
char *strsubst(char *, char, char);	

/* Return pointer code for pointer type characer passed. */
int getptrtype(char *);	

/* Return part of speech code for string passed */
int getpos(char *);		

/* Return synset type code for string passed. */
int getsstype(char *);		

/* Reconstruct synset from synset pointer and return ptr to buffer */
char *FmtSynset(SynsetPtr, int);	

/* Find string for 'searchstr' as it is in index file */
char *GetWNStr(char *, int);

/* Pass in string for POS, return corresponding integer value */
int StrToPos(char *);

/* Return synset for sense key passed. */
SynsetPtr GetSynsetForSense(char *);

/* Find offset of sense key in data file */
long GetDataOffset(char *);

/* Find polysemy (collins) count for sense key passed. */
int GetPolyCount(char *);

/* Return word part of sense key */
char *GetWORD(char *);

/* Return POS code for sense key passed. */
int GetPOS(char *);

/* Convert WordNet sense number passed of IndexPtr entry to sense key. */
char *WNSnsToStr(IndexPtr, int);

/* Search for string and/or baseform of word in database and return
   index structure for word if found in database. */
IndexPtr GetValidIndexPointer(char *, int);

/* Return sense number in database for word and lexsn passed. */
int GetWNSense(char *, char *);

SnsIndexPtr GetSenseIndex(char *);

char *GetOffsetForKey(unsigned int);
unsigned int GetKeyForOffset(char *);

char *SetSearchdir();

/* Return number of times sense is tagged */
int GetTagcnt(IndexPtr, int);

/*
** Wrapper functions for strstr that allow you to retrieve each
** occurance of a word within a longer string, not just the first.
**
** strstr_init is called with the same arguments as normal strstr,
** but does not return any value.
**
** strstr_getnext returns the position offset (not a pointer, as does
** normal strstr) of the next occurance, or -1 if none remain.
*/
void strstr_init (char *, char *);
int strstr_getnext (void);

/*** Binary search functions (binsearch.c) ***/

/* General purpose binary search function to search for key as first
   item on line in open file.  Item is delimited by space. */
char *bin_search(char *, FILE *);
char *read_index(long, FILE *);

/* Copy contents from one file to another. */
void copyfile(FILE *, FILE *);

/* Function to replace a line in a file.  Returns the original line,
   or NULL in case of error. */
char *replace_line(char *, char *, FILE *);

/* Find location to insert line at in file.  If line with this
   key is already in file, return NULL. */
char *insert_line(char *, char *, FILE *);

static int do_init_exc();
static int do_init();
static int strend(char *, char *);
static char *wordbase(char *, int);
static int hasprep(char *, int);
static char *exc_lookup(char *, int);
static char *morphprep(char *);

/* General error message handler - can be defined by interface.
   Default function provided in library returns -1 */

int display_message(char *);

/* Search.c Forward function declarations */

static void WNOverview(char *, int);
static void findverbgroups(IndexPtr);
static void add_relatives(int, IndexPtr, int, int);
static void free_rellist(void);
void printsynset(char *, SynsetPtr, char *, int, int, int, int);
static void printantsynset(SynsetPtr, char *, int, int);
static char *printant(int, SynsetPtr, int, char *, char *);
static void printbuffer(char *);
static void printsns(SynsetPtr, int);
static void printsense(SynsetPtr, int);
static void catword(char *, SynsetPtr, int, int, int);
static void printspaces(int, int);
static void printrelatives(IndexPtr, int);
static int HasHoloMero(IndexPtr, int);
static int HasPtr(SynsetPtr, int);
static int getsearchsense(SynsetPtr, int);
static int depthcheck(int, SynsetPtr);
static void getexample(char *, char *);
static int findexample(SynsetPtr);

char *getpartsname(int pos);
char *getSearchBuffer();
void setSearchBuffer(char *s);

void closefps(void);

#endif /*_WN_*/



